/*
* @Author: José Elías Gutierrez
* @Date:   2017-04-10 15:43:31
* @Last Modified by:   fry
* @Last Modified time: 2017-06-19 14:50:37
*/

'use strict';

/**
 * Ajax request to calculate and return price, and store booking session. Overrides plugin default function
 * Make sure its called after wceb class is loaded
 */
var jebe = {};
jQuery(document).ready(function () {

  (function ($) {
    jebe = {

      elements: {
        'start_hour': $('#start_hour'),
        'end_hour': $('#end_hour'),
        'inputStart': $('.datepicker_start').pickadate(),
        'inputEnd': $('.datepicker_end').pickadate(),
        'pickerStart': $('.datepicker_start').pickadate('picker'),
        'pickerEnd': $('.datepicker_end').pickadate('picker')
      },

      times: [],

      options: jebe_options,

      error_selector: 'div.wc_ebs_errors',

      error_tpl: '<div class="wc_ebs_errors"><ul class="woocommerce-error">\
							<li>{{message}}</li>\
					</ul>\
				</div>',

      last_error_message: null,

      disableHolidays: function (dates, pickerStartItem, pickerEndItem, mode) {

        var toDisable = ebac.getDates(data.disabled_dates[variationId], mode);

        $.each(dates, function (index, value) {

          var date = new Date(value + ' 00:00:00');
          var dateToDisable = [date.getFullYear(), date.getMonth(), date.getDate()];
          var time = date.getTime();
          dateToDisable['type'] = 'booked';

          toDisable['dates'].push(dateToDisable); // Array of dates object (day mode)
          toDisable['times'].push(time);

        });

        if (toDisable.length <= 0) {
          pickerStartItem.disable = [];
          pickerEndItem.disable = [];
          return false;
        }

        pickerStartItem.disable = ( mode === 'nights' ) ? dates.startDates : toDisable;

        // In "Nights" mode, end calendar dates have a one day offset
        pickerEndItem.disable = toDisable;

        this.times = toDisable;

        return false;
      },

      updatePickersAvailabilities: function () {
        var variation_id = $('.variations_form').find('input[name=variation_id]').val();
        var days_availability = jebe_days_availability[variation_id]['day'];

        $.each(days_availability, function (index, availability) {
          $('.picker__day--stock[data-pick="' + Date.parse(index + ' 00:00:00') + '"] > span').html(availability);
        });
      },

      isDateRangeComplete: function () {
        var startdate = jebe.elements.pickerStart.get('select', 'yyyy-mm-dd');
        var enddate = jebe.elements.pickerEnd.get('select', 'yyyy-mm-dd');
        var starttime = jebe.elements.start_hour.val();
        var endtime = jebe.elements.end_hour.val();

        return startdate !== '' && enddate !== '' && starttime !== '' && endtime !== '';
      },

      checkRange: function () {

        this.last_error_message = null;

        var startdate = jebe.elements.pickerStart.get('select', 'yyyy-mm-dd');
        var enddate = jebe.elements.pickerEnd.get('select', 'yyyy-mm-dd');

        var starttime = new Date(startdate + ' ' + jebe.elements.start_hour.val());
        var endtime = new Date(enddate + ' ' + jebe.elements.end_hour.val());

        // overlaps with non available datetime??
        var variation_id = $('.variations_form').find('input[name=variation_id]').val();
        var hour_availability = jebe_days_availability[variation_id]['hour'];
        var overlaps = false;

        $.each(hour_availability, function (index, availability) {
          if (availability == 0) {
            if (starttime <= new Date(index) && endtime >= new Date(index)) {
              overlaps = true;
            }
          }
        });

        if (overlaps) {
          this.last_error_message = this.options.messages.range_overlaps_unavailable_hours;
          return false;
        }

        if (this.isDateRangeComplete()) {
          // two or more hours selected
          if (Math.abs(endtime - starttime) / 3600000 >= 2) {
            if (starttime < new Date()) {
              this.last_error_message = this.options.messages.start_date_past;
              return false;
            }
            if (endtime <= starttime) {
              this.last_error_message = this.options.messages.endtime_not_less_than_starttime;
              return false;
            }
            return true;
          }
          this.last_error_message = this.options.messages.min_2_hours_error;
        }

        return false;

      },

      updateStartHourAvailabilities: function () {
        var variation_id = $('.variations_form').find('input[name=variation_id]').val();
        var hour_availability = jebe_days_availability[variation_id]['hour'];

        jebe.elements.start_hour.find('option').attr('disabled', false);

        var updated = false;

        var selected_date = jebe.elements.pickerStart.get('select', 'yyyy-mm-dd');

        $.each(hour_availability, function (index, availability) {
          if (index.split(' ')[0] == selected_date) {
            updated = true;
            if (availability == 0) {
              jebe.elements.start_hour.find('option[value="' + index.split(' ')[1] + '"]').attr('disabled', 'disabled');
            }
            jebe.elements.start_hour.find('option[value="' + index.split(' ')[1] + '"]').html(availability + ' - ' + index.split(' ')[1]);
          }
        });

        // if there is no availability info for selected day, there is full availability
        if (!updated) {
          var stock = $('.picker__day--stock[data-pick="' + Date.parse(selected_date + ' 00:00:00') + '"] > span').html();
          jebe.elements.start_hour.find('option').each(function (index, option) {
            if (this.value)
              $(this).html(stock + ' - ' + this.value);
          });
        }

        jebe.elements.start_hour.find('option').each(function (index, option) {
          if (jebe.disableWeekendDate(selected_date, this.value) === true)
            jebe.elements.start_hour.find('option[value="' + this.value + '"]').attr('disabled', 'disabled');
        });

      },

      updateEndHourAvailabilities: function () {
        var variation_id = $('.variations_form').find('input[name=variation_id]').val();
        var hour_availability = jebe_days_availability[variation_id]['hour'];

        jebe.elements.end_hour.find('option').attr('disabled', false);

        var updated = false;

        var selected_date = jebe.elements.pickerEnd.get('select', 'yyyy-mm-dd');

        $.each(hour_availability, function (index, availability) {
          if (index.split(' ')[0] == selected_date) {
            updated = true;
            if (availability == 0) {
              jebe.elements.end_hour.find('option[value="' + index.split(' ')[1] + '"]').attr('disabled', 'disabled');
            }
            jebe.elements.end_hour.find('option[value="' + index.split(' ')[1] + '"]').html(availability + ' - ' + index.split(' ')[1]);
          }
        });

        // if there is no availability info for selected day, there is full availability
        if (!updated) {
          var stock = $('.picker__day--stock[data-pick="' + Date.parse(selected_date + ' 00:00:00') + '"] > span').html();
          jebe.elements.end_hour.find('option').each(function (index, option) {
            if (this.value)
              $(this).html(stock + ' - ' + this.value);
          });
        }
      },

      disableWeekendDate: function (date, time) {
        var hour = time.split(':')[0];
        var auxDate = new Date(date.split('-')[0], date.split('-')[1] - 1, date.split('-')[2]);
        if (auxDate.getDay() == 6 && hour > 14) {
          return true;
        } else if (auxDate.getDay() == 0) {
          return true;
        }

        return false;
      },

      // add ele for calendar icons
      init: function () {
        $('.datepicker_start').after('<i class="jebe-calendar"></i>');
        $('.datepicker_end').after('<i class="jebe-calendar"></i>');
      }

    };

    jebe.init();

    // override pickerend set event to avoid limits on pickerstart
    $inputEnd = $('.datepicker_end').pickadate();
    pickerEnd = $inputEnd.pickadate('picker');

    pickerEnd.on({
      set: function (endTime) {

        // Reset min, max and disabled dates on other picker
        wceb.pickers.clearSecond('end', 'start', pickerStart);

        // If picker is cleared
        if (typeof endTime.clear !== 'undefined' && endTime.clear === null) {

          selectedDates['end'] = null;
          selectedDates['endFormat'] = null;

          // Clear session
          wceb.clearBookingSession();

          $('.single_add_to_cart_button').prop('disabled', true);

        }

        return false;
      }
    });

    jebe.elements.start_hour.on('change', function () {
      wceb.setPrice();
    });
    jebe.elements.end_hour.on('change', function () {
      wceb.setPrice();
    });

    jebe.elements.pickerStart.on('after_rendering', function () {
      $.each(jebe.times['times'], function (index, datePick) {
        var $day = $(".picker__day[data-pick='" + datePick + "']");
        $day.addClass('picker__day--disabled');
      });
      jebe.updatePickersAvailabilities();
    });

    jebe.elements.pickerEnd.on('after_rendering', function () {

      jebe.updatePickersAvailabilities();
    });

    jebe.elements.pickerStart.on('close', function () {
      jebe.updateStartHourAvailabilities();
    });

    jebe.elements.pickerEnd.on('close', function () {
      jebe.updateEndHourAvailabilities();
    });

    $('body').on('pickers_init', function (e, variation) {
      if (jebe.options.allow_holiday_bookings !== 'on') {

        jebe.disableHolidays(jebe.options.holidays, jebe.elements.pickerStart, jebe.elements.pickerEnd, wceb.calcMode);

      }

      return false;

    });

    // override wceb pickers init to avoid reloading dates after select changes
    wceb.pickers._init = wceb.pickers.init;

    var selectedDates = {
      start: '',
      end: '',
      start_hour: '',
      end_hour: '',
      clear: function () {
        this.start = '';
        this.end = '';
        this.start_hour = '';
        this.end_end = '';
      },
      set: function () {
        this.start = jebe.elements.pickerStart.get('select', 'yyyy-mm-dd');
        this.end = jebe.elements.pickerEnd.get('select', 'yyyy-mm-dd');
        this.start_hour = jebe.elements.start_hour.val();
        this.end_hour = jebe.elements.end_hour.val();
      },
      paste: function () {
        if (this.start != '')
          jebe.elements.pickerStart.set('select', new Date(this.start));
        if (this.end != '')
          jebe.elements.pickerEnd.set('select', new Date(this.end));
        if (this.start_hour != '')
          jebe.elements.start_hour.val(this.start_hour);
        if (this.end_hour != '')
          jebe.elements.end_hour.val(this.end_hour);
      }
    }

    // set today as default date if none is selected
    $('body').on('pickers_init', function (e, variation) {
      if (selectedDates.start == '') {
        jebe.elements.pickerStart.set('select', Date.now());
        jebe.elements.pickerEnd.set('select', Date.now());
        jebe.updateStartHourAvailabilities();
        jebe.updateEndHourAvailabilities();
      }
      // remove start picker max to allow free selection regardless of end picker date
    });

    wceb.pickers.init = function (restoreDates = true) {
      if (restoreDates)
        selectedDates.set();

      wceb.pickers._init();

      if (restoreDates)
        selectedDates.paste();
      else
        selectedDates.clear();
    };

    wceb.setPrice = function () {

      if (!jebe.isDateRangeComplete()) return false;

      if (!jebe.checkRange()) {

        $('.booking_price').find('.price').html('');

        var html = jebe.error_tpl;
        if (jebe.last_error_message != null)
          $(jebe.error_selector).replaceWith(html.replace('{{message}}', jebe.last_error_message));
        return false;
      }

      this.product_id = $('input[name=add-to-cart], button[name=add-to-cart]').val();
      this.variation_id = $('.variations_form').find('input[name=variation_id]').val();

      this.children = wceb.get.childrenIds();

      this.selectedDates = {};

      var format = $.fn.pickadate.defaults.format;

      // Start date
      this.selectedDates['startFormat'] = pickerStart.get('select', 'yyyy-mm-dd'), // yyyy-mm-dd
        this.selectedDates['start'] = pickerStart.get('select', format); // Nice format

      // End date
      this.selectedDates['endFormat'] = pickerEnd.get('select', 'yyyy-mm-dd'), // yyyy-mm-dd
        this.selectedDates['end'] = pickerEnd.get('select', format); // Nice format

      // WooCommerce Product Add-ons compatibility
      var additionalCost = wceb.get.additionalCosts('each');

      var extras = '';
      $('.addon-checkbox').each(function () {
        if ($(this).attr('checked') == 'checked') {
          if (extras == '') {
            extras += this.value;
          } else {
            extras += ', ' + this.value;
          }
        }
      });

      var data = {
        action: 'add_new_price',
        security: $('input[name="_wceb_nonce"]').val(),
        product_id: this.product_id,
        variation_id: this.variation_id,
        children: this.children,
        start: this.selectedDates['start'],
        end: this.selectedDates['end'],
        start_format: this.selectedDates['startFormat'],
        end_format: this.selectedDates['endFormat'],
        additional_cost: additionalCost,
        // Hours info
        start_hour: jebe.elements.start_hour.val(),
        end_hour: jebe.elements.end_hour.val(),
        extras: extras,
      };

      console.log(data);

      $('form.cart, form.bundle_form').fadeTo('400', '0.6').block({
        message: null,
        overlayCSS: {
          background: 'transparent',
          backgroundSize: '16px 16px',
          opacity: 0.6
        }
      });

      $('.woocommerce-variation-availability').html('');

      $.post(wceb_object.ajax_url, data, function (response) {

        $('.woocommerce-error, .woocommerce-message').remove();
        this.fragments = response.fragments;
        this.errors = response.errors;

        jebe.max_bookable_qty = response.max_bookable_qty;
        if (response.max_bookable_qty == 0) {
          $('.woocommerce-variation-availability').html('<div class="clear"></div><div><p>' + wc_add_to_cart_variation_params.i18n_unavailable_text + '</p></div>');
          $('.booking_price').find('.price').html('');
          $('form.cart, form.bundle_form').stop(true).css('opacity', '1').unblock();
          return false;
        } else {
          var message = jebe.options.messages.max_bookable_qty;
          $('.woocommerce-variation-availability').html('<div class="clear"></div><div><p> ' + message.replace('{{qty}}', response.max_bookable_qty) + ' </p></div>');
        }

        // If error, reset pickers
        if (this.errors) {

          $.each(this.errors, function (key, value) {
            $(key).replaceWith(value);
          });

          wceb.pickers.init();
          pickerStart.render();
          pickerEnd.render();

          // Unblock
          $('form.cart, form.bundle_form').stop(true).css('opacity', '1').unblock();

          return false;

        }

        if (this.fragments) {

          $.each(this.fragments, function (key, value) {
            $(key).replaceWith(value);
          });

          // Get quantity selected
          if ($('.cart').find('input[name="quantity"]').size()) {
            var qty_selected = parseFloat($('.cart').find('input[name="quantity"]').val());
          } else {
            var qty_selected = 1;
          }

          // Multiply booking price by quantity selected
          var new_price = wceb.formatPrice(this.fragments.booking_price * qty_selected);
          var price_html = '<span class="woocommerce-Price-amount amount">' + new_price + '</span>';

          // If the product is on sale
          if (this.fragments.booking_regular_price !== '') {
            var new_regular_price = wceb.formatPrice(this.fragments.booking_regular_price * qty_selected);
            var price_html = '<del><span class="woocommerce-Price-amount amount">' + new_regular_price + '</span></del> <ins><span class="woocommerce-Price-amount amount">' + new_price + '</span></ins>';
            $('.booking_price').attr('data-booking_regular_price', this.fragments.booking_regular_price);
          }

          // Update price
          $('.booking_price').attr('data-booking_price', this.fragments.booking_price)
            .find('.price').html(price_html);

          wceb.session = this.fragments.session;

        }

        $('body').trigger('update_price', [data, response]);

        // Unblock
        $('form.cart, form.bundle_form').stop(true).css('opacity', '1').unblock();

      });
    }

    $('body').on('change', '.booking__extra', function () {
      console.log($(this));
    });
  }(jQuery));

});
